from typing import Protocol

from attr import frozen

from ..bounds.execution.execution import ExecutionBounded, ExecutionBoundedlike


class Executionlike(ExecutionBoundedlike, Protocol):

    # an id for the execution
    id : str

@frozen
class Execution(ExecutionBounded, ExecutionBoundedlike):

    # an id for the the exeuction
    id : str


from typing import Optional, Protocol, Sequence

from pydantic import BaseModel
import requests
from dotenv import load_dotenv
import os

load_dotenv()

class Stadiumlike(BaseModel):
    StadiumID: int
    Active: bool
    Name: str
    Address: Optional[str]
    City: str
    State: str
    Zip: Optional[str]
    Country: Optional[str]
    Capacity: Optional[int]
    GeoLat: Optional[float]
    GeoLong: Optional[float]


class Teamlike(BaseModel):
    TeamID: int
    Key: str
    Active: bool
    School: str
    Name: str
    ApRank: Optional[int]
    Wins: Optional[int]
    Losses: Optional[int]
    ConferenceWins: Optional[int]
    ConferenceLosses: Optional[int]
    GlobalTeamID: int
    ConferenceID: Optional[int]
    Conference: Optional[str]
    TeamLogoUrl: str
    ShortDisplayName: str
    Stadium: Optional[Stadiumlike]



class Stadium(BaseModel):
    StadiumID: int
    Active: bool
    Name: str
    Address: Optional[str]
    City: str
    State: str
    Zip: Optional[str]
    Country: Optional[str]
    Capacity: Optional[int]
    GeoLat: Optional[float]
    GeoLong: Optional[float]


class Team(BaseModel):
    TeamID: int
    Key: str
    Active: bool
    School: str
    Name: str
    ApRank: Optional[int]
    Wins: Optional[int]
    Losses: Optional[int]
    ConferenceWins: Optional[int]
    ConferenceLosses: Optional[int]
    GlobalTeamID: int
    ConferenceID: Optional[int]
    Conference: Optional[str]
    TeamLogoUrl: str
    ShortDisplayName: str
    Stadium: Optional[Stadiumlike]


def get_teams()->Sequence[Teamlike]:
    """Gets all teams

    Returns:
        Sequence[Teamlike]: gets info about all teams
    """
    domain = os.getenv("SPORTS_DATA_DOMAIN")
    return [
        Team.parse_obj(team)
        for team in requests.get(
        f"{domain}/v3/cbb/scores/json/teams",
        params={
            "key" : os.getenv("SPORTS_DATA_KEY")
        }
    ).json()]

def get_team(id: str) -> Optional[Teamlike]:
    """Gets a team by id.

    Args:
        id (str): the id of the team.

    Returns:
        Optional[Teamlike]: _description_
    """
    for team in get_teams():
        if team.team_id == id:
            return team


from datetime import datetime
from typing import Any, Callable, Dict, List, Protocol, TypeVar, cast, Optional

import requests
from dotenv import load_dotenv
from pydantic import BaseModel
from json import dumps
import os

load_dotenv()

class TeamSeasonStatslike(BaseModel):
    StatID : int
    TeamID : int
    SeasonType : int
    Season : int
    Name : str
    Team : Optional[str]
    Wins : Optional[int]
    Losses : Optional[int]
    ConferenceWins : Optional[int]
    ConferenceLosses : Optional[int]
    GlobalTeamID: int
    Possessions : float
    Updated : datetime
    Games : int
    FantasyPoints : float
    Minutes : int
    FieldGoalsMade : int
    FieldGoalsAttempted : int
    FieldGoalsPercentage : float
    EffectiveFieldGoalsPercentage : float
    TwoPointersMade : int
    TwoPointersAttempted : int
    TwoPointersPercentage : int
    ThreePointersMade : int
    ThreePointersAttempted : int
    ThreePointersPercentage : float
    FreeThrowsMade : int
    FreeThrowsAttempted : int
    FreeThrowsPercentage : float
    OffensiveRebounds : int
    DefensiveRebounds : int
    Rebounds : int
    OffensiveReboundsPercentage : Optional[float]
    DefensiveReboundsPercentage : Optional[float]
    TotalReboundsPercentage : Optional[float]
    Assists : int
    Steals : int
    BlockedShots : int
    Turnovers : int
    PersonalFouls : int
    Points: int
    TrueShootingAttempts : float
    TrueShootingPercentage : float
    PlayerEfficiencyRating : Optional[float]
    AssistsPercentage :  Optional[float]
    StealsPercentage :  Optional[float]
    BlocksPercentage :  Optional[float]
    TurnOversPercentage :  Optional[float]
    UsageRatePercentage :  Optional[float]
    FantasyPointsFanDuel :  Optional[float]
    FantasyPointsDraftKings :  Optional[float]
    FantasyPointsYahoo :  Optional[float]
    
class TeamSeasonStats(BaseModel):
    StatID : int
    TeamID : int
    SeasonType : int
    Season : int
    Name : str
    Team : Optional[str]
    Wins : Optional[int]
    Losses : Optional[int]
    ConferenceWins : Optional[int]
    ConferenceLosses : Optional[int]
    GlobalTeamID: int
    Possessions : float
    Updated : datetime
    Games : int
    FantasyPoints : float
    Minutes : int
    FieldGoalsMade : int
    FieldGoalsAttempted : int
    FieldGoalsPercentage : float
    EffectiveFieldGoalsPercentage : float
    TwoPointersMade : int
    TwoPointersAttempted : int
    TwoPointersPercentage : int
    ThreePointersMade : int
    ThreePointersAttempted : int
    ThreePointersPercentage : float
    FreeThrowsMade : int
    FreeThrowsAttempted : int
    FreeThrowsPercentage : float
    OffensiveRebounds : int
    DefensiveRebounds : int
    Rebounds : int
    OffensiveReboundsPercentage : Optional[float]
    DefensiveReboundsPercentage : Optional[float]
    TotalReboundsPercentage : Optional[float]
    Assists : int
    Steals : int
    BlockedShots : int
    Turnovers : int
    PersonalFouls : int
    Points: int
    TrueShootingAttempts : float
    TrueShootingPercentage : float
    PlayerEfficiencyRating : Optional[float]
    AssistsPercentage :  Optional[float]
    StealsPercentage :  Optional[float]
    BlocksPercentage :  Optional[float]
    TurnOversPercentage :  Optional[float]
    UsageRatePercentage :  Optional[float]
    FantasyPointsFanDuel :  Optional[float]
    FantasyPointsDraftKings :  Optional[float]
    FantasyPointsYahoo :  Optional[float]

def get_team_season_stats_by_date(date : datetime) -> List[TeamSeasonStats]:
    """Gets games by date directly from sportsdataio

    Args:
        date (datetime): is the date in question.

    Returns:
        List[TeamGameStatsByDatelike]: are the games by date.
    """
    domain = os.getenv("SPORTS_DATA_DOMAIN")
    json = requests.get(
        f"{domain}/v3/cbb/scores/json/TeamSeasonStats/{date.year}",
        params={
            "key" : os.getenv("SPORTS_DATA_KEY")
        }
    ).json()
    return [TeamSeasonStats.parse_obj(obj) for obj in json]


from datetime import datetime
from typing import Any, Callable, Dict, List, Protocol, TypeVar, cast, Optional
from .team import Stadiumlike

import requests
from dotenv import load_dotenv
from pydantic import BaseModel
import os

load_dotenv()


class Periodlike(BaseModel):
    PeriodID: int
    GameID: int
    Number: int
    Name: str
    Type: str
    AwayScore: Optional[int]
    HomeScore: Optional[int]

class GameByDatelike(BaseModel):
    GameID: int
    Season: int
    SeasonType: int
    Status: str
    Day: datetime
    DateTime: Optional[datetime]
    AwayTeam: str
    HomeTeam: str
    AwayTeamID: int
    HomeTeamID: int
    AwayTeamScore: Optional[int]
    HomeTeamScore: Optional[int]
    Updated: datetime
    Period: Optional[str]
    TimeRemainingMinutes: Optional[int]
    TimeRemainingSeconds: Optional[int]
    PointSpread: Optional[float]
    OverUnder: Optional[float]
    AwayTeamMoneyLine: Optional[int]
    HomeTeamMoneyLine: Optional[int]
    GlobalGameID: int
    GlobalAwayTeamID: int
    GlobalHomeTeamID: int
    TournamentID: Optional[str]
    Bracket: Optional[str]
    Round: Optional[str]
    AwayTeamSeed: Optional[int]
    HomeTeamSeed: Optional[int]
    AwayTeamPreviousGameID: Optional[int]
    HomeTeamPreviousGameID: Optional[int]
    AwayTeamPreviousGlobalGameID: Optional[int]
    HomeTeamPreviousGloablGameID: Optional[int]
    TournamentDisplayOrder: Optional[int]
    TournamentDisplayOrderForHomeTeam: str
    IsClosed: bool
    GameEndDateTime: Optional[datetime]
    HomeRotationNumber: Optional[int]
    AwayRotationNumber: Optional[int]
    TopTeamPreviousGameID: Optional[str]
    BottomTeamPreviousGameID: Optional[str]
    Channel: Optional[str]
    NeutralVenue: Optional[bool]
    AwayPointSpreadPayout: Optional[str]
    HomePointSpreadPayout: Optional[str]
    OverPayout: Optional[str]
    UnderPayout: Optional[str]
    DateTimeUTC: Optional[datetime]
    Stadium: Optional[Stadiumlike]
    Periods: List[Periodlike]
    
class Period(BaseModel):
    PeriodID: int
    GameID: int
    Number: int
    Name: str
    Type: str
    AwayScore: Optional[int]
    HomeScore: Optional[int]

class GameByDate(BaseModel):
    GameID: int
    Season: int
    SeasonType: int
    Status: str
    Day: datetime
    DateTime: Optional[datetime]
    AwayTeam: str
    HomeTeam: str
    AwayTeamID: int
    HomeTeamID: int
    AwayTeamScore: Optional[int]
    HomeTeamScore: Optional[int]
    Updated: datetime
    Period: Optional[str]
    TimeRemainingMinutes: Optional[int]
    TimeRemainingSeconds: Optional[int]
    PointSpread: Optional[float]
    OverUnder: Optional[float]
    AwayTeamMoneyLine: Optional[int]
    HomeTeamMoneyLine: Optional[int]
    GlobalGameID: int
    GlobalAwayTeamID: int
    GlobalHomeTeamID: int
    TournamentID: Optional[str]
    Bracket: Optional[str]
    Round: Optional[str]
    AwayTeamSeed: Optional[int]
    HomeTeamSeed: Optional[int]
    AwayTeamPreviousGameID: Optional[int]
    HomeTeamPreviousGameID: Optional[int]
    AwayTeamPreviousGlobalGameID: Optional[int]
    HomeTeamPreviousGloablGameID: Optional[int]
    TournamentDisplayOrder: Optional[int]
    TournamentDisplayOrderForHomeTeam: Optional[str]
    IsClosed: Optional[bool]
    GameEndDateTime: Optional[datetime]
    HomeRotationNumber: Optional[int]
    AwayRotationNumber: Optional[int]
    TopTeamPreviousGameID: Optional[str]
    BottomTeamPreviousGameID: Optional[str]
    Channel: Optional[str]
    NeutralVenue: Optional[bool]
    AwayPointSpreadPayout: Optional[str]
    HomePointSpreadPayout: Optional[str]
    OverPayout: Optional[str]
    UnderPayout: Optional[str]
    DateTimeUTC: Optional[datetime]
    Stadium: Optional[Stadiumlike]
    Periods: List[Periodlike]


def get_games(date : datetime) -> List[GameByDatelike]:
    """Gets games by date directly from sportsdataio

    Args:
        date (datetime): is the date in question.

    Returns:
        List[GameByDatelike]: are the games by date.
    """
    domain = os.getenv("SPORTS_DATA_DOMAIN")
    json = requests.get(
        f"{domain}/v3/cbb/scores/json/GamesByDate/{date.year}-{date.month}-{date.day}",
        params={
            "key" : os.getenv("SPORTS_DATA_KEY")
        }
    ).json()
    return [GameByDate.parse_obj(g) for g in json]


from datetime import datetime
from typing import Any, Callable, Dict, List, Protocol, TypeVar, cast, Optional

import requests
from dotenv import load_dotenv
from pydantic import BaseModel
from json import dumps
import os

load_dotenv()

class TeamGameStatsByDatelike(BaseModel):
    StatID : int
    TeamID : int
    SeasonType : int
    Season : int
    Name : str
    Team : str
    Wins : Optional[int]
    Losses : Optional[int]
    ConferenceWins : Optional[int]
    ConferenceLosses : Optional[int]
    GlobalTeamID : Optional[int]
    Possessions : Optional[int]
    GameID : int
    OpponentID : int
    Opponent : str
    Day : Optional[datetime]
    DateTime : Optional[datetime]
    HomeOrAway : str
    IsGameOver : bool
    GlobalGameID : int
    GlobalOpponentID : int
    Updated : Optional[datetime]
    Games : int
    FantasyPoints : Optional[float]
    Minutes : Optional[int]
    FieldGoalsMade : int
    FieldGoalsAttempted : int
    FieldGoalsPercentage : int
    EffectiveFieldGoalsPercentage : float
    TwoPointersMade : int
    TwoPointersAttempted : int
    TwoPointersPercentage : float
    ThreePointersMade : int
    ThreePointersAttempted : int
    ThreePointersPercentage : float
    FreeThrowsMade : int
    FreeThrowsAttempted : int
    FreeThrowsPercentage : float
    OffensiveRebounds : int
    DefensiveRebounds : int
    Rebounds : int
    OffensiveReboundsPercentage : Optional[float]
    DefensiveReboundsPercentage : Optional[float]
    TotalReboundsPercentage : Optional[float]
    Assists : int
    Steals : int
    BlockedShots : int
    Turnovers : int
    PersonalFouls : int
    Points : Optional[int]
    TrueShootingAttempts : float
    TrueShootingPercentage : float
    PlayerEfficiencyRating : Optional[float]
    AssistsPercentage : Optional[float]
    StealsPercentage : Optional[float]
    BlocksPercentage : Optional[float]
    TurnOversPercentage : Optional[float]
    UsageRatePercentage : Optional[float]
    FantasyPointsFanDuel : Optional[float]
    FantasyPointsDraftKings : Optional[float]
    FantasyPointsYahoo : Optional[float]

class TeamGameStatsByDate(BaseModel):
    StatID : int
    TeamID : int
    SeasonType : int
    Season : int
    Name : str
    Team : str
    Wins : Optional[int]
    Losses : Optional[int]
    ConferenceWins : Optional[int]
    ConferenceLosses : Optional[int]
    GlobalTeamID : Optional[int]
    Possessions : Optional[int]
    GameID : int
    OpponentID : int
    Opponent : str
    Day : Optional[datetime]
    DateTime : Optional[datetime]
    HomeOrAway : str
    IsGameOver : bool
    GlobalGameID : int
    GlobalOpponentID : int
    Updated : Optional[datetime]
    Games : int
    FantasyPoints : Optional[float]
    Minutes : Optional[int]
    FieldGoalsMade : int
    FieldGoalsAttempted : int
    FieldGoalsPercentage : int
    EffectiveFieldGoalsPercentage : float
    TwoPointersMade : int
    TwoPointersAttempted : int
    TwoPointersPercentage : float
    ThreePointersMade : int
    ThreePointersAttempted : int
    ThreePointersPercentage : float
    FreeThrowsMade : int
    FreeThrowsAttempted : int
    FreeThrowsPercentage : float
    OffensiveRebounds : int
    DefensiveRebounds : int
    Rebounds : int
    OffensiveReboundsPercentage : Optional[float]
    DefensiveReboundsPercentage : Optional[float]
    TotalReboundsPercentage : Optional[float]
    Assists : int
    Steals : int
    BlockedShots : int
    Turnovers : int
    PersonalFouls : int
    Points : Optional[int]
    TrueShootingAttempts : float
    TrueShootingPercentage : float
    PlayerEfficiencyRating : Optional[float]
    AssistsPercentage : Optional[float]
    StealsPercentage : Optional[float]
    BlocksPercentage : Optional[float]
    TurnOversPercentage : Optional[float]
    UsageRatePercentage : Optional[float]
    FantasyPointsFanDuel : Optional[float]
    FantasyPointsDraftKings : Optional[float]
    FantasyPointsYahoo : Optional[float]


def get_game_stats_by_date(date : datetime) -> List[TeamGameStatsByDatelike]:
    """Gets games by date directly from sportsdataio

    Args:
        date (datetime): is the date in question.

    Returns:
        List[TeamGameStatsByDatelike]: are the games by date.
    """
    domain = os.getenv("SPORTS_DATA_DOMAIN")
    json = requests.get(
        f"{domain}/v3/cbb/scores/json/TeamGameStatsByDate/{date.year}-{date.month}-{date.day}",
        params={
            "key" : os.getenv("SPORTS_DATA_KEY")
        }
    ).json()
    return [TeamGameStatsByDate.parse_obj(entry) for entry in json]


from .context.bounds.bounds import execution, session, universal, user
from .context.execution.execution import Execution
from .context.session.session import Session
from .context.user.user import User
from .context.context.context import Context, root
from .model.model import Model, poll, secs, mins, hours, days, months, dow, Init
from .modifiers.state import private, public
from .event import Event
from .sdk import spiodirect


import unittest
import time
from fastapi import FastAPI
from libutil.sync.deasync import deasync

def timeit(method):
    def timed(*args, **kw):
        ts = time.time()
        result = method(*args, **kw)
        te = time.time()
        if 'log_time' in kw:
            name = kw.get('log_name', method.__name__.upper())
            kw['log_time'][name] = int((te - ts) * 1000)
        else:
            print('%r  %2.2f ms' % (method.__name__, (te - ts) * 1000))
        return result
    return timed
 
class MYTestCase(unittest.TestCase):
    """Checks that the get_image_colors methods work appropriately.
    """

    @deasync
    async def test_can_create_callable(self):
        """Experimenting with fast api
        """
        app = FastAPI()
        async def add(a : int, b : int)->int:
            return a + b

        @app.post(add.__name__)
        async def inner(a : int, b : int)->int:
            return a + b
    